import { CommonModule } from '@angular/common';
import { takeUntil } from 'rxjs/operators';
import { EventEmitter, Injectable, ChangeDetectorRef, Directive, Input, ViewContainerRef, NgModule } from '@angular/core';
import { BehaviorSubject, isObservable, Subject } from 'rxjs';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const values = new WeakMap();
/**
 * @param {?} instance
 * @return {?}
 */
function valueMap(instance) {
    /** @type {?} */
    let _valueMap = values.get(instance);
    if (!_valueMap) {
        _valueMap = new Map();
        values.set(instance, _valueMap);
    }
    return _valueMap;
}
/**
 * @param {?} target
 * @param {?} propertyKey
 * @param {?=} callback
 * @return {?}
 */
function redefineSimpleProperty(target, propertyKey, callback) {
    Object.defineProperty(target, propertyKey, {
        configurable: true,
        enumerable: true,
        /**
         * @param {?} value
         * @return {?}
         */
        set(value) {
            valueMap(this).set(propertyKey, value);
            if (callback) {
                callback(value);
            }
        },
        /**
         * @return {?}
         */
        get() {
            return valueMap(this).get(propertyKey);
        }
    });
}
/**
 * @param {?} target
 * @param {?} propertyKey
 * @param {?} descriptor
 * @param {?=} callback
 * @return {?}
 */
function redefineAccessorProperty(target, propertyKey, descriptor, callback) {
    Object.defineProperty(target, propertyKey, {
        configurable: descriptor.configurable,
        enumerable: descriptor.enumerable,
        /**
         * @param {?} value
         * @return {?}
         */
        set(value) {
            if (!descriptor.set) {
                throw new Error(`Property ${propertyKey} doesn't have a setter and cannot be written`);
            }
            descriptor.set.call(this, value);
            if (!descriptor.get) {
                callback(value);
            }
            else {
                callback(descriptor.get());
            }
        },
        /**
         * @return {?}
         */
        get() {
            if (!descriptor.get) {
                throw new Error(`Property ${propertyKey} doesn't have a getter and cannot be read`);
            }
            return descriptor.get.call(this);
        }
    });
}
/**
 * @param {?} target
 * @param {?} propertyKey
 * @return {?}
 */
function getPropDescriptor(target, propertyKey) {
    /** @type {?} */
    let originalDescriptor = Reflect.getOwnPropertyDescriptor(target, propertyKey);
    /** @type {?} */
    let setter = originalDescriptor && originalDescriptor.set;
    /** @type {?} */
    let getter = originalDescriptor && originalDescriptor.get;
    /** @type {?} */
    const value = originalDescriptor ? originalDescriptor.value : target[propertyKey];
    if (!setter || ((/** @type {?} */ (target))).__lookupSetter__(propertyKey)) {
        setter = ((/** @type {?} */ (target))).__lookupSetter__(propertyKey);
    }
    if (!getter || ((/** @type {?} */ (target))).__lookupGetter__(propertyKey)) {
        getter = ((/** @type {?} */ (target))).__lookupGetter__(propertyKey);
    }
    if (!originalDescriptor && setter && getter) {
        originalDescriptor = {
            configurable: true,
            enumerable: true,
            set: setter.bind(target),
            get: getter.bind(target)
        };
    }
    return {
        originalDescriptor: originalDescriptor,
        value,
        setter,
        getter
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBindInputsService {
    /**
     * BindInputs Directive
     * @param {?} directive
     * @return {?}
     */
    bindInputs(directive) {
        /** @type {?} */
        const inputs = this.getInputs(directive);
        /** @type {?} */
        const excludeInputs = (Array.isArray(directive.excludeInputs)
            ? directive.excludeInputs
            : [directive.excludeInputs]).map(key => key.toUpperCase());
        /** @type {?} */
        const includeInputs = (Array.isArray(directive.includeInputs)
            ? directive.includeInputs
            : [directive.includeInputs]).map(key => key.toUpperCase());
        inputs.parentKeys
            .filter(parentKey => (includeInputs.length === 0 && excludeInputs.indexOf(parentKey.toUpperCase()) === -1) ||
            (includeInputs.length !== 0 && includeInputs.indexOf(parentKey.toUpperCase()) !== -1))
            .forEach(parentKey => {
            inputs.keys.forEach(key => {
                if (this.checkInputToBind(directive, parentKey, key)) {
                    this.bindInput(directive, parentKey, key);
                }
            });
        });
    }
    /**
     * @param {?} directive
     * @return {?}
     */
    bindObservableInputs(directive) {
        /** @type {?} */
        const inputs = this.getInputs(directive);
        /** @type {?} */
        const excludeInputs = (Array.isArray(directive.excludeInputs)
            ? directive.excludeInputs
            : [directive.excludeInputs]).map(key => key.toUpperCase());
        /** @type {?} */
        const includeInputs = (Array.isArray(directive.includeInputs)
            ? directive.includeInputs
            : [directive.includeInputs]).map(key => key.toUpperCase());
        inputs.parentKeys
            .filter(parentKey => (includeInputs.length === 0 && excludeInputs.indexOf(parentKey.toUpperCase()) === -1) ||
            (includeInputs.length !== 0 && includeInputs.indexOf(parentKey.toUpperCase()) !== -1))
            .forEach(parentKey => {
            inputs.keys.forEach(key => {
                if (this.checkObservableInputToBind(directive, parentKey, key)) {
                    this.bindObservableInput(directive, parentKey, key);
                }
            });
        });
    }
    /**
     * Inputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    checkKeyNameToInputBind(directive, parentKey, key) {
        return parentKey === key;
    }
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    checkInputToBind(directive, parentKey, key) {
        /** @type {?} */
        const parentValue = getPropDescriptor(directive.parentComponent, parentKey).value;
        /** @type {?} */
        const value = getPropDescriptor(directive.component, key).value;
        return (directive.usedInputs.indexOf(parentKey) === -1 &&
            this.checkKeyNameToInputBind(directive, parentKey, key) &&
            !isObservable(parentValue) &&
            !isObservable(value));
    }
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    bindInput(directive, parentKey, key) {
        directive.usedInputs.push(parentKey);
        /** @type {?} */
        const descriptor = getPropDescriptor(directive.parentComponent, parentKey);
        /** @type {?} */
        const currentValue = descriptor.value;
        delete directive.parentComponent[parentKey];
        if (descriptor !== undefined && descriptor.setter !== undefined && descriptor.getter !== undefined) {
            redefineAccessorProperty(directive.parentComponent, parentKey, descriptor.originalDescriptor, (newValue) => directive.bindValue(key, newValue));
        }
        else {
            redefineSimpleProperty(directive.parentComponent, parentKey, (newValue) => directive.bindValue(key, newValue));
        }
        directive.parentComponent[parentKey] = currentValue;
    }
    /**
     * Observable Inputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    checkKeyNameToObservableInputBind(directive, parentKey, key) {
        return parentKey === `${key}$`;
    }
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    checkObservableInputToBind(directive, parentKey, key) {
        /** @type {?} */
        const parentValue = getPropDescriptor(directive.parentComponent, parentKey).value;
        /** @type {?} */
        const value = getPropDescriptor(directive.component, key).value;
        return (directive.usedInputs.indexOf(parentKey) === -1 &&
            this.checkKeyNameToObservableInputBind(directive, parentKey, key) &&
            isObservable(parentValue) &&
            !isObservable(value));
    }
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    bindObservableInput(directive, parentKey, key) {
        directive.usedInputs.push(parentKey);
        /** @type {?} */
        const descriptor = getPropDescriptor(directive.parentComponent, parentKey);
        /** @type {?} */
        const isBehaviorSubject = descriptor.value instanceof BehaviorSubject;
        /** @type {?} */
        const currentValue = descriptor.value;
        /** @type {?} */
        const behaviorSubjectValue = isBehaviorSubject ? descriptor.value.getValue() : undefined;
        delete directive.parentComponent[parentKey];
        if (descriptor !== undefined && descriptor.setter !== undefined && descriptor.getter !== undefined) {
            redefineAccessorProperty(directive.parentComponent, parentKey, descriptor.originalDescriptor, (newValue) => {
                if (directive.parentComponent[parentKey + '_subscription']) {
                    ((/** @type {?} */ (directive.parentComponent[parentKey + '_subscription']))).unsubscribe();
                }
                directive.parentComponent[parentKey + '_subscription'] = newValue
                    .pipe(takeUntil(directive.destroyed$))
                    .subscribe(value => directive.bindValue(key, value));
            });
        }
        else {
            redefineSimpleProperty(directive.parentComponent, parentKey, (newValue) => {
                if (directive.parentComponent[parentKey + '_subscription']) {
                    ((/** @type {?} */ (directive.parentComponent[parentKey + '_subscription']))).unsubscribe();
                }
                directive.parentComponent[parentKey + '_subscription'] = newValue
                    .pipe(takeUntil(directive.destroyed$))
                    .subscribe(value => directive.bindValue(key, value));
            });
        }
        directive.parentComponent[parentKey] = currentValue;
        if (isBehaviorSubject) {
            currentValue.next(behaviorSubjectValue);
        }
    }
    /**
     * Utils
     * @param {?} directive
     * @return {?}
     */
    getInputs(directive) {
        /** @type {?} */
        const data = {
            parentKeys: [
                ...Object.keys(directive.parentComponent),
                ...Object.keys(directive.parentComponent.__proto__),
                ...Object.keys(directive.parentComponent.__proto__ ? directive.parentComponent.__proto__.__proto__ : [])
            ],
            keys: [
                ...Object.keys(directive.component ? directive.component : []).filter(key => !(getPropDescriptor(directive.component, key).value instanceof EventEmitter))
            ]
        };
        return data;
    }
}
NgxBindInputsService.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BindInputsDirective {
    /**
     * @param {?} _viewContainerRef
     * @param {?} _ngxBindInputsService
     * @param {?} _ref
     */
    constructor(_viewContainerRef, _ngxBindInputsService, _ref) {
        this._viewContainerRef = _viewContainerRef;
        this._ngxBindInputsService = _ngxBindInputsService;
        this._ref = _ref;
        this.excludeInputs = [];
        this.includeInputs = [];
        this.usedInputs = [];
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.component = this._viewContainerRef['_data'].componentView.component;
        this.parentComponent = ((/** @type {?} */ (this._viewContainerRef)))._view.context;
        this.inputs = this._ngxBindInputsService.getInputs(this);
        this._ngxBindInputsService.bindInputs(this);
        this._ngxBindInputsService.bindObservableInputs(this);
        if (localStorage && localStorage.getItem('debug_ngx-bind-io') === 'true') {
            console.log(this.inputs, this);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    bindValue(key, value) {
        this.component[key] = value;
        this._ref.markForCheck();
    }
}
BindInputsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[bindInputs]'
            },] }
];
/** @nocollapse */
BindInputsDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: NgxBindInputsService },
    { type: ChangeDetectorRef }
];
BindInputsDirective.propDecorators = {
    excludeInputs: [{ type: Input }],
    includeInputs: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} functionToCheck
 * @return {?}
 */
function isFunction(functionToCheck) {
    return functionToCheck && {}.toString.call(functionToCheck) === '[object Function]';
}
/**
 * @param {?} derivedCtor
 * @param {?} baseCtors
 * @return {?}
 */
function applyMixins(derivedCtor, baseCtors) {
    baseCtors.forEach(baseCtor => {
        Object.getOwnPropertyNames(baseCtor.prototype).forEach(name => {
            derivedCtor.prototype[name] = baseCtor.prototype[name];
        });
    });
    return derivedCtor;
}
/**
 * @param {?} baseCtors
 * @return {?}
 */
function Mixin(baseCtors) {
    return function (derivedCtor) {
        baseCtors.forEach(baseCtor => {
            Object.getOwnPropertyNames(baseCtor.prototype).forEach(name => {
                derivedCtor.prototype[name] = baseCtor.prototype[name];
            });
        });
    };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBindOutputsService {
    /**
     * BindOutputs Directive
     * @param {?} directive
     * @return {?}
     */
    bindOutputs(directive) {
        /** @type {?} */
        const outputs = this.getOutputs(directive);
        /** @type {?} */
        const excludeOutputs = (Array.isArray(directive.excludeOutputs)
            ? directive.excludeOutputs
            : [directive.excludeOutputs]).map(key => key.toUpperCase());
        /** @type {?} */
        const includeOutputs = (Array.isArray(directive.includeOutputs)
            ? directive.includeOutputs
            : [directive.includeOutputs]).map(key => key.toUpperCase());
        outputs.parentKeys
            .filter(parentKey => (includeOutputs.length === 0 && excludeOutputs.indexOf(parentKey.toUpperCase()) === -1) ||
            (includeOutputs.length !== 0 && includeOutputs.indexOf(parentKey.toUpperCase()) !== -1))
            .forEach(parentKey => {
            outputs.keys.forEach(key => {
                if (this.checkOutputToBind(directive, parentKey, key)) {
                    this.bindOutput(directive, parentKey, key);
                }
            });
        });
    }
    /**
     * Outputs
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    checkKeyNameToOutputBind(directive, parentKey, key) {
        /** @type {?} */
        const outputs = this.getOutputs(directive);
        /** @type {?} */
        const keyWithFirstUpperLetter = key.length > 0 ? key.charAt(0).toUpperCase() + key.substr(1) : key;
        return ((parentKey === `on${keyWithFirstUpperLetter}` &&
            outputs.parentKeys.indexOf(`on${keyWithFirstUpperLetter}Click`) === -1) ||
            parentKey === `on${keyWithFirstUpperLetter}Click`);
    }
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    checkOutputToBind(directive, parentKey, key) {
        return directive.usedOutputs.indexOf(key) === -1 && this.checkKeyNameToOutputBind(directive, parentKey, key);
    }
    /**
     * @param {?} directive
     * @param {?} parentKey
     * @param {?} key
     * @return {?}
     */
    bindOutput(directive, parentKey, key) {
        directive.usedOutputs.push(key);
        directive.component[key].subscribe(value => directive.parentComponent[parentKey](value));
    }
    /**
     * Utils
     * @param {?} directive
     * @return {?}
     */
    getOutputs(directive) {
        /** @type {?} */
        const data = {
            parentKeys: [
                ...Object.keys(directive.parentComponent).filter(parentKey => isFunction(directive.parentComponent[parentKey])),
                ...Object.keys(directive.parentComponent.__proto__).filter(parentKey => isFunction(getPropDescriptor(directive.parentComponent.__proto__, parentKey).value)),
                ...Object.keys(directive.parentComponent.__proto__ ? directive.parentComponent.__proto__.__proto__ : []).filter(parentKey => isFunction(getPropDescriptor(directive.parentComponent.__proto__.__proto__, parentKey).value))
            ],
            keys: [
                ...Object.keys(directive.component ? directive.component : []).filter(key => getPropDescriptor(directive.component, key).value instanceof EventEmitter)
            ]
        };
        return data;
    }
}
NgxBindOutputsService.decorators = [
    { type: Injectable }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BindIODirective {
    /**
     * @param {?} _viewContainerRef
     * @param {?} _ngxBindInputsService
     * @param {?} _ngxBindOutputsService
     * @param {?} _ref
     */
    constructor(_viewContainerRef, _ngxBindInputsService, _ngxBindOutputsService, _ref) {
        this._viewContainerRef = _viewContainerRef;
        this._ngxBindInputsService = _ngxBindInputsService;
        this._ngxBindOutputsService = _ngxBindOutputsService;
        this._ref = _ref;
        this.excludeInputs = [];
        this.includeInputs = [];
        this.excludeOutputs = [];
        this.includeOutputs = [];
        this.usedInputs = [];
        this.usedOutputs = [];
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.component = this._viewContainerRef['_data'].componentView.component;
        this.parentComponent = ((/** @type {?} */ (this._viewContainerRef)))._view.context;
        this.inputs = this._ngxBindInputsService.getInputs(this);
        this.outputs = this._ngxBindOutputsService.getOutputs(this);
        this._ngxBindInputsService.bindInputs(this);
        this._ngxBindInputsService.bindObservableInputs(this);
        this._ngxBindOutputsService.bindOutputs(this);
        if (localStorage && localStorage.getItem('debug_ngx-bind-io') === 'true') {
            console.log(this.inputs, this.outputs, this);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    bindValue(key, value) {
        this.component[key] = value;
        this._ref.markForCheck();
    }
}
BindIODirective.decorators = [
    { type: Directive, args: [{
                selector: '[bindIO]'
            },] }
];
/** @nocollapse */
BindIODirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: NgxBindInputsService },
    { type: NgxBindOutputsService },
    { type: ChangeDetectorRef }
];
BindIODirective.propDecorators = {
    excludeInputs: [{ type: Input }],
    includeInputs: [{ type: Input }],
    excludeOutputs: [{ type: Input }],
    includeOutputs: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BindOutputsDirective {
    /**
     * @param {?} _viewContainerRef
     * @param {?} _ngxBindOutputsService
     * @param {?} _ref
     */
    constructor(_viewContainerRef, _ngxBindOutputsService, _ref) {
        this._viewContainerRef = _viewContainerRef;
        this._ngxBindOutputsService = _ngxBindOutputsService;
        this._ref = _ref;
        this.excludeOutputs = [];
        this.includeOutputs = [];
        this.usedOutputs = [];
        this.destroyed$ = new Subject();
    }
    /**
     * @return {?}
     */
    ngAfterContentInit() {
        this.component = this._viewContainerRef['_data'].componentView.component;
        this.parentComponent = ((/** @type {?} */ (this._viewContainerRef)))._view.context;
        this.outputs = this._ngxBindOutputsService.getOutputs(this);
        this._ngxBindOutputsService.bindOutputs(this);
        if (localStorage && localStorage.getItem('debug_ngx-bind-io') === 'true') {
            console.log(this.outputs, this);
        }
    }
    /**
     * @return {?}
     */
    ngOnDestroy() {
        this.destroyed$.next(true);
        this.destroyed$.complete();
    }
}
BindOutputsDirective.decorators = [
    { type: Directive, args: [{
                selector: '[bindOutputs]'
            },] }
];
/** @nocollapse */
BindOutputsDirective.ctorParameters = () => [
    { type: ViewContainerRef },
    { type: NgxBindOutputsService },
    { type: ChangeDetectorRef }
];
BindOutputsDirective.propDecorators = {
    excludeOutputs: [{ type: Input }],
    includeOutputs: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NgxBindIOModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: NgxBindIOModule,
            providers: [NgxBindInputsService, NgxBindOutputsService]
        };
    }
}
NgxBindIOModule.decorators = [
    { type: NgModule, args: [{
                imports: [CommonModule],
                declarations: [BindInputsDirective, BindOutputsDirective, BindIODirective],
                exports: [BindInputsDirective, BindOutputsDirective, BindIODirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NgxBindIOModule, BindInputsDirective, BindIODirective, BindOutputsDirective, NgxBindInputsService, NgxBindOutputsService, redefineSimpleProperty, redefineAccessorProperty, getPropDescriptor, isFunction, applyMixins, Mixin };

//# sourceMappingURL=ngx-bind-io.js.map